@@ -5,7 +5,7 @@
 
     Base lexer classes.
 
-    :copyright: 2006 by Georg Brandl.
+    :copyright: 2006-2007 by Georg Brandl.
     :license: BSD, see LICENSE for more details.
 """
 import re
@@ -15,8 +15,11 @@ try:
 except NameError:
     from sets import Set as set
 
+from pygments.filter import apply_filters, Filter
+from pygments.filters import get_filter_by_name
 from pygments.token import Error, Text, Other, _TokenType
-from pygments.util import get_bool_opt, get_int_opt, make_analysator
+from pygments.util import get_bool_opt, get_int_opt, get_list_opt, \
+     make_analysator
 
 
 __all__ = ['Lexer', 'RegexLexer', 'ExtendedRegexLexer', 'DelegatingLexer',
@@ -81,6 +84,10 @@ class Lexer(object):
         self.stripall = get_bool_opt(options, 'stripall', False)
         self.tabsize = get_int_opt(options, 'tabsize', 0)
         self.encoding = options.get('encoding', 'latin1')
+        # self.encoding = options.get('inencoding', None) or self.encoding
+        self.filters = []
+        for filter_ in get_list_opt(options, 'filters', ()):
+            self.add_filter(filter_)
 
     def __repr__(self):
         if self.options:
@@ -89,6 +96,14 @@ class Lexer(object):
         else:
             return '<pygments.lexers.%s>' % self.__class__.__name__
 
+    def add_filter(self, filter_, **options):
+        """
+        Add a new stream filter to this lexer.
+        """
+        if not isinstance(filter_, Filter):
+            filter_ = get_filter_by_name(filter_, **options)
+        self.filters.append(filter_)
+
     def analyse_text(text):
         """
         Has to return a float between ``0`` and ``1`` that indicates
@@ -103,11 +118,14 @@ class Lexer(object):
         it's the same as if the return values was ``0.0``.
         """
 
-    def get_tokens(self, text):
+    def get_tokens(self, text, unfiltered=False):
         """
-        Return an iterable of (tokentype, value) pairs generated from ``text``.
+        Return an iterable of (tokentype, value) pairs generated from
+        `text`. If `unfiltered` is set to `True`, the filtering mechanism
+        is bypassed even if filters are defined.
 
-        Also preprocess the text, i.e. expand tabs and strip it if wanted.
+        Also preprocess the text, i.e. expand tabs and strip it if
+        wanted and applies registered filters.
         """
         if isinstance(text, unicode):
             text = u'\n'.join(text.splitlines())
@@ -115,16 +133,18 @@ class Lexer(object):
             text = '\n'.join(text.splitlines())
             if self.encoding == 'guess':
                 try:
-                    text = text.decode('utf-8-sig')
+                    text = text.decode('utf-8')
+                    if text.startswith(u'\ufeff'):
+                        text = text[len(u'\ufeff'):]
                 except UnicodeDecodeError:
                     text = text.decode('latin1')
             elif self.encoding == 'chardet':
                 try:
                     import chardet
                 except ImportError:
-                    raise ImportError('To enable chardet encoding guessing, please '
-                                      'install the chardet library from '
-                                      'http://chardet.feedparser.org/')
+                    raise ImportError('To enable chardet encoding guessing, '
+                                      'please install the chardet library '
+                                      'from http://chardet.feedparser.org/')
                 enc = chardet.detect(text)
                 text = text.decode(enc['encoding'])
             else:
@@ -138,8 +158,13 @@ class Lexer(object):
         if not text.endswith('\n'):
             text += '\n'
 
-        for i, t, v in self.get_tokens_unprocessed(text):
-            yield t, v
+        def streamer():
+            for i, t, v in self.get_tokens_unprocessed(text):
+                yield t, v
+        stream = streamer()
+        if not unfiltered:
+            stream = apply_filters(stream, self.filters, self)
+        return stream
 
     def get_tokens_unprocessed(self, text):
         """
@@ -271,8 +296,23 @@ def using(_other, **kwargs):
     """
     Callback that processes the match with a different lexer.
 
-    The keyword arguments are forwarded to the lexer.
+    The keyword arguments are forwarded to the lexer, except `state` which
+    is handled separately.
+
+    `state` specifies the state that the new lexer will start in, and can
+    be an enumerable such as ('root', 'inline', 'string') or a simple
+    string which is assumed to be on top of the root state.
+
+    Note: For that to work, `_other` must not be an `ExtendedRegexLexer`.
     """
+    gt_kwargs = {}
+    if 'state' in kwargs:
+        s = kwargs.pop('state')
+        if isinstance(s, (list, tuple)):
+            gt_kwargs['stack'] = s
+        else:
+            gt_kwargs['stack'] = ('root', s)
+
     if _other is this:
         def callback(lexer, match, ctx=None):
             # if keyword arguments are given the callback
@@ -284,7 +324,7 @@ def using(_other, **kwargs):
             else:
                 lx = lexer
             s = match.start()
-            for i, t, v in lx.get_tokens_unprocessed(match.group()):
+            for i, t, v in lx.get_tokens_unprocessed(match.group(), **gt_kwargs):
                 yield i + s, t, v
             if ctx:
                 ctx.pos = match.end()
@@ -295,7 +335,7 @@ def using(_other, **kwargs):
             lx = _other(**kwargs)
 
             s = match.start()
-            for i, t, v in lx.get_tokens_unprocessed(match.group()):
+            for i, t, v in lx.get_tokens_unprocessed(match.group(), **gt_kwargs):
                 yield i + s, t, v
             if ctx:
                 ctx.pos = match.end()
@@ -308,26 +348,30 @@ class RegexLexerMeta(LexerMeta):
     self.tokens on the first instantiation.
     """
 
-    def _process_state(cls, state):
+    def _process_state(cls, unprocessed, processed, state):
         assert type(state) is str, "wrong state name %r" % state
         assert state[0] != '#', "invalid state name %r" % state
-        if state in cls._tokens:
-            return cls._tokens[state]
-        tokens = cls._tokens[state] = []
+        if state in processed:
+            return processed[state]
+        tokens = processed[state] = []
         rflags = cls.flags
-        for tdef in cls.tokens[state]:
+        for tdef in unprocessed[state]:
             if isinstance(tdef, include):
                 # it's a state reference
                 assert tdef != state, "circular state reference %r" % state
-                tokens.extend(cls._process_state(str(tdef)))
+                tokens.extend(cls._process_state(unprocessed, processed, str(tdef)))
                 continue
 
             assert type(tdef) is tuple, "wrong rule def %r" % tdef
 
-            rex = re.compile(tdef[0], rflags)
+            try:
+                rex = re.compile(tdef[0], rflags).match
+            except Exception, err:
+                raise ValueError("uncompilable regex %r in state %r of %r: %s" %
+                                 (tdef[0], state, cls, err))
 
             assert type(tdef[1]) is _TokenType or callable(tdef[1]), \
-                   'token type must be simple type or callable, not %r' % tdef[1]
+                   'token type must be simple type or callable, not %r' % (tdef[1],)
 
             if len(tdef) == 2:
                 new_state = None
@@ -337,7 +381,7 @@ class RegexLexerMeta(LexerMeta):
                     # an existing state
                     if tdef2 == '#pop':
                         new_state = -1
-                    elif tdef2 in cls.tokens:
+                    elif tdef2 in unprocessed:
                         new_state = (tdef2,)
                     elif tdef2 == '#push':
                         new_state = tdef2
@@ -352,13 +396,14 @@ class RegexLexerMeta(LexerMeta):
                     itokens = []
                     for istate in tdef2:
                         assert istate != state, 'circular state ref %r' % istate
-                        itokens.extend(cls._process_state(istate))
-                    cls._tokens[new_state] = itokens
+                        itokens.extend(cls._process_state(unprocessed,
+                                                          processed, istate))
+                    processed[new_state] = itokens
                     new_state = (new_state,)
                 elif isinstance(tdef2, tuple):
                     # push more than one state
                     for state in tdef2:
-                        assert state in cls.tokens, \
+                        assert state in unprocessed, \
                                'unknown new state ' + state
                     new_state = tdef2
                 else:
@@ -366,12 +411,22 @@ class RegexLexerMeta(LexerMeta):
             tokens.append((rex, tdef[1], new_state))
         return tokens
 
+    def process_tokendef(cls, name, tokendefs=None):
+        processed = cls._all_tokens[name] = {}
+        tokendefs = tokendefs or cls.tokens[name]
+        for state in tokendefs.keys():
+            cls._process_state(tokendefs, processed, state)
+        return processed
+
     def __call__(cls, *args, **kwds):
         if not hasattr(cls, '_tokens'):
-            cls._tokens = {}
+            cls._all_tokens = {}
             cls._tmpname = 0
-            for state in cls.tokens.keys():
-                cls._process_state(state)
+            if hasattr(cls, 'token_variants') and cls.token_variants:
+                # don't process yet
+                pass
+            else:
+                cls._tokens = cls.process_tokendef('', cls.tokens)
 
         return type.__call__(cls, *args, **kwds)
 
@@ -414,12 +469,14 @@ class RegexLexer(Lexer):
         ``stack`` is the inital stack (default: ``['root']``)
         """
         pos = 0
+        tokendefs = self._tokens
         statestack = list(stack)
-        statetokens = self._tokens[statestack[-1]]
+        statetokens = tokendefs[statestack[-1]]
         while 1:
-            for rex, action, new_state in statetokens:
-                m = rex.match(text, pos)
+            for rexmatch, action, new_state in statetokens:
+                m = rexmatch(text, pos)
                 if m:
+                    # print rex.pattern
                     if type(action) is _TokenType:
                         yield pos, action, m.group()
                     else:
@@ -437,7 +494,7 @@ class RegexLexer(Lexer):
                             statestack.append(statestack[-1])
                         else:
                             assert False, "wrong state def: %r" % new_state
-                        statetokens = self._tokens[statestack[-1]]
+                        statetokens = tokendefs[statestack[-1]]
                     break
             else:
                 try:
@@ -445,7 +502,7 @@ class RegexLexer(Lexer):
                         # at EOL, reset state to "root"
                         pos += 1
                         statestack = ['root']
-                        statetokens = self._tokens['root']
+                        statetokens = tokendefs['root']
                         yield pos, Text, u'\n'
                         continue
                     yield pos, Error, text[pos]
@@ -480,16 +537,17 @@ class ExtendedRegexLexer(RegexLexer):
         Split ``text`` into (tokentype, text) pairs.
         If ``context`` is given, use this lexer context instead.
         """
+        tokendefs = self._tokens
         if not context:
             ctx = LexerContext(text, 0)
-            statetokens = self._tokens['root']
+            statetokens = tokendefs['root']
         else:
             ctx = context
-            statetokens = self._tokens[ctx.stack[-1]]
+            statetokens = tokendefs[ctx.stack[-1]]
             text = ctx.text
         while 1:
-            for rex, action, new_state in statetokens:
-                m = rex.match(text, ctx.pos, ctx.end)
+            for rexmatch, action, new_state in statetokens:
+                m = rexmatch(text, ctx.pos, ctx.end)
                 if m:
                     if type(action) is _TokenType:
                         yield ctx.pos, action, m.group()
@@ -499,7 +557,7 @@ class ExtendedRegexLexer(RegexLexer):
                             yield item
                         if not new_state:
                             # altered the state stack?
-                            statetokens = self._tokens[ctx.stack[-1]]
+                            statetokens = tokendefs[ctx.stack[-1]]
                     # CAUTION: callback must set ctx.pos!
                     if new_state is not None:
                         # state transition
@@ -512,7 +570,7 @@ class ExtendedRegexLexer(RegexLexer):
                             ctx.stack.append(ctx.stack[-1])
                         else:
                             assert False, "wrong state def: %r" % new_state
-                        statetokens = self._tokens[ctx.stack[-1]]
+                        statetokens = tokendefs[ctx.stack[-1]]
                     break
             else:
                 try:
@@ -522,7 +580,7 @@ class ExtendedRegexLexer(RegexLexer):
                         # at EOL, reset state to "root"
                         ctx.pos += 1
                         ctx.stack = ['root']
-                        statetokens = self._tokens['root']
+                        statetokens = tokendefs['root']
                         yield ctx.pos, Text, u'\n'
                         continue
                     yield ctx.pos, Error, text[ctx.pos]
@@ -543,7 +601,7 @@ def do_insertions(insertions, tokens):
 
     The result is a combined token stream.
 
-    FIXME: The indices yielded by this function are not correct!
+    TODO: clean up the code here.
     """
     insertions = iter(insertions)
     try:
@@ -554,17 +612,36 @@ def do_insertions(insertions, tokens):
             yield item
         return
 
+    realpos = None
     insleft = True
+
+    # iterate over the token stream where we want to insert
+    # the tokens from the insertion list.
     for i, t, v in tokens:
+        # first iteration. store the postition of first item
+        if realpos is None:
+            realpos = i
         oldi = 0
         while insleft and i + len(v) >= index:
-            yield i, t, v[oldi:index-i]
-            for item in itokens:
-                yield item
-            oldi = index-i
+            tmpval = v[oldi:index - i]
+            yield realpos, t, tmpval
+            realpos += len(tmpval)
+            for it_index, it_token, it_value in itokens:
+                yield realpos, it_token, it_value
+                realpos += len(it_value)
+            oldi = index - i
             try:
                 index, itokens = insertions.next()
             except StopIteration:
                 insleft = False
                 break  # not strictly necessary
-        yield i, t, v[oldi:]
+        yield realpos, t, v[oldi:]
+        realpos += len(v) - oldi
+
+    # leftover tokens
+    if insleft:
+        # no normal tokens, set realpos to zero
+        realpos = realpos or 0
+        for p, t, v in itokens:
+            yield realpos, t, v
+            realpos += len(v)
