@@ -5,7 +5,7 @@
 
     Utility functions.
 
-    :copyright: 2006 by Georg Brandl.
+    :copyright: 2006-2007 by Georg Brandl.
     :license: BSD, see LICENSE for more details.
 """
 import re
@@ -18,37 +18,61 @@ doctype_lookup_re = re.compile(r'''(?smx)
      [a-zA-Z_][a-zA-Z0-9]*\s+
      [a-zA-Z_][a-zA-Z0-9]*\s+
      "[^"]*")
-     [^>]+>
+     [^>]*>
 ''')
 tag_re = re.compile(r'<(.+?)(\s.*?)?>.*?</\1>(?uism)')
 
 
+class ClassNotFound(ValueError):
+    """
+    If one of the get_*_by_* functions didn't find a matching class.
+    """
+
+
 class OptionError(Exception):
     pass
 
 
+def get_choice_opt(options, optname, allowed, default=None):
+    string = options.get(optname, default)
+    if string not in allowed:
+        raise OptionError('Value for option %s must be one of %s' %
+                          (optname, ', '.join(map(str, allowed))))
+    return string
+
+
 def get_bool_opt(options, optname, default=None):
     string = options.get(optname, default)
     if isinstance(string, bool):
         return string
+    elif isinstance(string, int):
+        return bool(string)
+    elif not isinstance(string, basestring):
+        raise OptionError('Invalid type %r for option %s; use '
+                          '1/0, yes/no, true/false, on/off' % (
+                          string, optname))
     elif string.lower() in ('1', 'yes', 'true', 'on'):
         return True
     elif string.lower() in ('0', 'no', 'false', 'off'):
         return False
     else:
         raise OptionError('Invalid value %r for option %s; use '
-                          '1/0, yes/no, true/false, on/off' %
-                          string, optname)
+                          '1/0, yes/no, true/false, on/off' % (
+                          string, optname))
 
 
 def get_int_opt(options, optname, default=None):
     string = options.get(optname, default)
     try:
         return int(string)
+    except TypeError:
+        raise OptionError('Invalid type %r for option %s; you '
+                          'must give an integer value' % (
+                          string, optname))
     except ValueError:
         raise OptionError('Invalid value %r for option %s; you '
-                          'must give an integer value' %
-                          string, optname)
+                          'must give an integer value' % (
+                          string, optname))
 
 
 def get_list_opt(options, optname, default=None):
@@ -58,9 +82,21 @@ def get_list_opt(options, optname, default=None):
     elif isinstance(val, (list, tuple)):
         return list(val)
     else:
-        raise OptionError('Invalid value %r for option %s; you '
-                          'must give a list value' %
-                          val, optname)
+        raise OptionError('Invalid type %r for option %s; you '
+                          'must give a list value' % (
+                          val, optname))
+
+
+def docstring_headline(obj):
+    if not obj.__doc__:
+        return ''
+    res = []
+    for line in obj.__doc__.strip().splitlines():
+        if line.strip():
+            res.append(" " + line.strip())
+        else:
+            break
+    return ''.join(res).lstrip()
 
 
 def make_analysator(f):
